@@ -5,81 +5,57 @@ from django.utils.safestring import mark_safe
 from django_datatables.datatables import DatatableTable
 from django_datatables.plugins.reorder import Reorder
 from django_datatables.reorder_datatable import OrderedDatatable
-from django_menus.menu import HtmlMenu
+
+CARD_TYPE_STANDARD = 1
+CARD_TYPE_DATATABLE = 2
+CARD_TYPE_ORDERED_DATATABLE = 3
+CARD_TYPE_HTML = 4
 
 
 class CardBase:
     template_types = {'default': 'cards/standard/default.html',
-                      'table': 'cards/standard/table.html'}
-    CARD_TYPE_STANDARD = 1
-    CARD_TYPE_DATATABLE = 2
-    CARD_TYPE_ORDERED_DATATABLE = 3
-    CARD_TYPE_HTML = 4
+                      'table': 'cards/standard/table.html',
+                      'datatable': 'cards/standard/datatable.html',
+                      'html': 'cards/standard/html.html'}
 
     ajax_commands = ['datatable']
 
-    datatable_order_field = 'order'
-    datatable_model = None
+    template_defaults = {CARD_TYPE_STANDARD: 'default',
+                         CARD_TYPE_DATATABLE: 'datatable',
+                         CARD_TYPE_ORDERED_DATATABLE: 'datatable',
+                         CARD_TYPE_HTML: 'html'}
 
-    def __init__(self, view=None):
-        self.detail_cards = {}
-        self.current_card = None
-        self.tables = {}
-        self.request = None
-        if view is None:
-            self.view = self
-        else:
-            self.view = view
-        super().__init__()
+    def __init__(self, request, code, view=None, details_object=None, title=None, menu=None, template_name=None,
+                 group_type=CARD_TYPE_STANDARD,
+                 show_created_modified_dates=False, extra_card_context=None, **kwargs):
 
-    def add_detail_card(self, code, title, menu=None,
-                        created_modified_dates=None,
-                        group_type=CARD_TYPE_STANDARD,
-                        details_object=None,
-                        extra_card_context=None,
-                        template_name='default',
-                        **kwargs):
+        self.code = code
+        self.view = view
+        self.details_object = details_object
+        self.request = request
+        self.group_type = group_type
+        self.show_created_modified_dates = show_created_modified_dates
 
-        if menu is not None:
-            details_menu = HtmlMenu(self.request, 'button_group').add_items(*menu)
-        else:
-            details_menu = None
+        self.rows = []
+        self.title = title
+        self.created_modified_dates = self.get_created_modified_dates(details_object=details_object)
+        self.menu = menu
+        self.extra_card_context = extra_card_context
+        self.template_name = template_name
+        self.extra_card_info = {}
+        self.add_extra_card_info(extra_info=self.extra_card_info, group_type=self.group_type, **kwargs)
 
-        if group_type == self.CARD_TYPE_STANDARD:
-            self.detail_cards[code] = {'rows': [],
-                                       'title': title,
-                                       'created_modified_dates': created_modified_dates,
-                                       'menu': details_menu,
-                                       'type': group_type,
-                                       'details_object': details_object,
-                                       'extra_card_context': extra_card_context,
-                                       'group_type': group_type,
-                                       'template_name': template_name}
-        elif group_type in (self.CARD_TYPE_DATATABLE, self.CARD_TYPE_ORDERED_DATATABLE):
-            datatable = kwargs.get('datatable')
-            datatable_model = kwargs.get('datatable_model')
-            self.detail_cards[code] = {'title': title,
-                                       'created_modified_dates': created_modified_dates,
-                                       'menu': details_menu,
-                                       'type': group_type,
-                                       'details_object': details_object,
-                                       'extra_card_context': extra_card_context,
-                                       'group_type': group_type,
-                                       'datatable': datatable,
-                                       'datatable_model': datatable_model,
-                                       'template_name': template_name}
-        elif group_type == self.CARD_TYPE_HTML:
-            html = kwargs.get('html')
-            self.detail_cards[code] = {'html': html,
-                                       'title': title,
-                                       'created_modified_dates': created_modified_dates,
-                                       'menu': details_menu,
-                                       'type': group_type,
-                                       'details_object': details_object,
-                                       'extra_card_context': extra_card_context,
-                                       'group_type': group_type,
-                                       'template_name': template_name}
-        self.current_card = code
+        self.process_data()
+
+    # noinspection PyMethodMayBeStatic
+    def add_extra_card_info(self, extra_info, group_type,  **kwargs):
+        if group_type in (CARD_TYPE_DATATABLE, CARD_TYPE_ORDERED_DATATABLE):
+            extra_info['datatable'] = kwargs.get('datatable')
+            extra_info['datatable_model'] = kwargs.get('datatable_model')
+            if group_type == CARD_TYPE_ORDERED_DATATABLE:
+                extra_info['order_field'] = kwargs.get('order_field', 'order')
+        elif group_type == CARD_TYPE_HTML:
+            extra_info['html'] = kwargs.get('html')
 
     def add_boolean_entry(self, value, label=None):
         if value:
@@ -114,7 +90,7 @@ class CardBase:
                     css_class = css_types.get(entry_len, 'col-sm-6')
 
                 entry = {'type': 'multiple', 'entries': entries, 'css_class': css_class}
-            self.detail_cards[self.current_card]['rows'].append(entry)
+            self.rows.append(entry)
 
     def add_entry(self, value=None, field=None, label=None, css_class=None, default='N/A', link=None,
                   hidden=False, hidden_if_blank_or_none=False, html_override=None, value_method=None):
@@ -131,12 +107,12 @@ class CardBase:
                                          value_method=value_method,
                                          )
         if entry is not None:
-            self.detail_cards[self.current_card]['rows'].append({'type': 'standard', 'entries': [entry]})
+            self.rows.append({'type': 'standard', 'entries': [entry]})
 
     def get_field_value(self, value, field, label):
 
         if value is None and field is not None:
-            details_object = self.detail_cards[self.current_card]['details_object']
+            details_object = self.details_object
             if details_object is not None:
                 value = details_object
                 old_value = None
@@ -192,38 +168,36 @@ class CardBase:
                      'multiple_lines': multiple_lines, 'link': link}
             return entry
 
-    def get_details_data(self, details_object, group_type):
-        if (group_type in (self.CARD_TYPE_DATATABLE, self.CARD_TYPE_ORDERED_DATATABLE) and
-                self.detail_cards[self.current_card]['datatable'] is None):
+    def process_data(self):
+        if (self.group_type in (CARD_TYPE_DATATABLE, CARD_TYPE_ORDERED_DATATABLE) and
+                self.extra_card_info['datatable'] is None):
 
-            datatable_model = self.detail_cards[self.current_card]['datatable_model']
-            if group_type == self.CARD_TYPE_ORDERED_DATATABLE:
-                table = self.add_ordered_table(model=datatable_model, order_field=self.datatable_order_field)
+            datatable_model = self.extra_card_info['datatable_model']
+            table_id = self.code
+            if self.extra_card_info['datatable'] is None:
+                if self.group_type == CARD_TYPE_ORDERED_DATATABLE:
+
+                    table = self.add_ordered_table(table_id=table_id,
+                                                   model=datatable_model,
+                                                   order_field=self.extra_card_info['order_field'])
+                else:
+                    table = self.add_table(table_id=table_id, model=datatable_model)
+                self.extra_card_info['datatable'] = table
             else:
-                table = self.add_table(model=datatable_model)
+                table = self.extra_card_info['datatable']
             setup_table_field = 'setup_table'
-            if self.current_card is not None:
-                field_setup_table_field = f'setup_table_{self.current_card}'
+            if self.code is not None:
+                field_setup_table_field = f'setup_table_{self.code}'
                 if hasattr(self.view, field_setup_table_field):
                     setup_table_field = field_setup_table_field
 
-            getattr(self.view, setup_table_field)(details_object=details_object, table=table)
-            self.detail_cards[self.current_card]['datatable'] = table
-
-    def get_details_menu(self, details_object):
-        return []
-
-    def get_details_title(self, details_object):
-        return "Details"
-
-    def get_group_type(self, details_object):
-        return self.CARD_TYPE_STANDARD
+            getattr(self.view, setup_table_field)(details_object=self.details_object, table=table)
 
     def add_table(self, model, table_id=None):
         table = DatatableTable(table_id=table_id, model=model, view=self.view)
-        self.tables[table_id] = table
+        self.extra_card_info['datatable'] = table
         table.ajax_data = False
-        return self.tables[table_id]
+        return table
 
     def add_ordered_table(self, model, order_field, table_id=None):
         table = OrderedDatatable(table_id=table_id,
@@ -232,28 +206,26 @@ class CardBase:
                                  order_field=order_field)
         table.add_plugin(Reorder)
         table.ajax_data = False
-        self.tables[table_id] = table
-        return self.tables[table_id]
+        self.extra_card_info['datatable'] = table
+        return table
 
-    def setup_table(self, table, details_object):
-        pass
-
-    @staticmethod
-    def get_created_modified_dates(details_object):
-        if details_object and hasattr(details_object, 'created') and hasattr(details_object, 'modified'):
+    def get_created_modified_dates(self, details_object):
+        if (self.show_created_modified_dates and details_object and
+                hasattr(details_object, 'created') and hasattr(details_object, 'modified')):
             return details_object.created, details_object.modified
         return None
 
     def datatable_sort(self, **kwargs):
-        order_field = self.datatable_order_field
+        order_field = self.extra_card_info['order_field']
         ids = [x[1] for x in kwargs['sort']]
-        current_sort = dict(self.datatable_model.objects.filter(id__in=ids).values_list('id', order_field))
+        current_sort = dict(self.extra_card_info['datatable_model'].objects.filter(id__in=ids).
+                            values_list('id', order_field))
         for s in kwargs['sort']:
             if current_sort[s[1]] != s[0]:
-                o = self.datatable_model.objects.get(id=s[1])
+                o = self.extra_card_info['datatable_model'].objects.get(id=s[1])
                 setattr(o, order_field, s[0])
                 o.save()
-        return self.command_response('')
+        return self.command_response('null')
 
     @staticmethod
     def label_from_field(field):
@@ -284,23 +256,23 @@ class CardBase:
             elif isinstance(arg, (list, tuple)):
                 self.add_row(*arg)
 
-    def _render_cards(self):
+    def render(self):
+        extra_card_context = self.extra_card_context
 
-        data = ''
-        for card_name, card in self.detail_cards.items():
-            extra_card_context = card['extra_card_context']
+        context = {'card': self,
+                   'request': self.request,
+                   'card_types': {'standard': CARD_TYPE_STANDARD,
+                                  'datatable': CARD_TYPE_DATATABLE,
+                                  'ordered_datatable': CARD_TYPE_ORDERED_DATATABLE,
+                                  'html': CARD_TYPE_HTML}}
 
-            context = {'card': card,
-                       'request': self.request,
-                       'card_types': {'standard': self.CARD_TYPE_STANDARD,
-                                      'datatable': self.CARD_TYPE_DATATABLE,
-                                      'ordered_datatable': self.CARD_TYPE_ORDERED_DATATABLE,
-                                      'html': self.CARD_TYPE_HTML}}
+        if extra_card_context is not None:
+            context = {**context, **extra_card_context}
 
-            if extra_card_context is not None:
-                context = {**context, **extra_card_context}
+        template_name = self.template_name
+        if template_name is None:
+            template_name = self.template_defaults.get(self.group_type)
 
-            template_name = card['template_name']
-            template = self.template_types.get(template_name, template_name)
-            data += render_to_string(template, context)
+        template = self.template_types.get(template_name, template_name)
+        data = render_to_string(template, context)
         return mark_safe(data)
