@@ -1,15 +1,15 @@
 from ajax_helpers.mixins import AjaxHelpers
+from ajax_helpers.utils import random_string
 from django.shortcuts import get_object_or_404
-from django.template.loader import render_to_string
 from django.utils.safestring import mark_safe
 from django.views.generic import TemplateView
 from django_menus.menu import MenuMixin
 from django_modals.modals import ModalException
 
-from cards.base import CardBase
+from cards.base import CardBase, CARD_TYPE_STANDARD
 
 
-class CardListMixin(CardBase, TemplateView):
+class CardListMixin(TemplateView):
     template_name = 'cards/list/list.html'
     list_class = 'col-sm-5 col-md-4 col-lg-3 float-left'
     details_class = 'col-sm-7 col-md-8 col-lg-9 float-left'
@@ -18,6 +18,10 @@ class CardListMixin(CardBase, TemplateView):
     list_title = ''
     menu_display = ''
 
+    datatable_model = None
+
+    card_cls = CardBase
+
     model = None
 
     def split_slug(self, kwargs):
@@ -39,6 +43,7 @@ class CardListMixin(CardBase, TemplateView):
         self.list_entries = []
         super().__init__()
         self.slug = {}
+        self.cards = {}
 
     # noinspection PyMethodMayBeStatic
     def process_slug_kwargs(self):
@@ -96,51 +101,88 @@ class CardListMixin(CardBase, TemplateView):
     def get_details_object(self, pk):
         return get_object_or_404(self.model, pk=pk)
 
+    def get_details_menu(self, details_object):
+        return []
+
+    def get_details_title(self, details_object):
+        return 'Details'
+
+    def get_group_type(self, details_object):
+        return CARD_TYPE_STANDARD
+
     def get_list_menu(self):
         return []
 
-    def set_card_template(self, template_name):
-        self.detail_cards[self.current_card]['template_name'] = template_name
+    def get_card_template(self):
+        return None
+
+    def get_show_created_modified_dates(self, details_object):
+        return True
+
+    def get_extra_card_kwargs(self, details_object):
+        return {}
 
     def button_details_html(self, extra_card_context=None, **kwargs):
         details_object = self.get_details_object(pk=kwargs['entry_id'])
 
-        menu = self.get_details_menu(details_object=details_object)
-        title = self.get_details_title(details_object=details_object)
-
-        group_type = self.get_group_type(details_object=details_object)
-        created_modified_dates = self.get_created_modified_dates(details_object=details_object)
-        self.add_detail_card(code=None,
-                             title=title,
-                             menu=menu,
-                             created_modified_dates=created_modified_dates,
-                             group_type=group_type,
-                             details_object=details_object,
-                             datatable_model=self.datatable_model,
-                             extra_card_context=extra_card_context)
-
-        self.set_card_template('default')
-
-        self.get_details_data(details_object=details_object, group_type=group_type)
-
-        # data = render_to_string(self.details_template_name, {'groups': self.detail_groups,
-        #                                                      'request': self.request,
-        #                                                      'group_types': {'standard': self.GROUP_TYPE_STANDARD,
-        #                                                                      'datatable': self.GROUP_TYPE_DATATABLE,
-        #                                                                      'html': self.GROUP_TYPE_HTML}})
+        if hasattr(self, 'setup_cards'):
+            self.setup_cards(details_object=details_object)
+        else:
+            self.add_main_card(details_object=details_object, extra_card_context=extra_card_context)
 
         data = self._render_cards()
         return self.command_response('html', selector='#details_card', html=data)
 
-    def add_html_group(self, code, title, template_name, context, extra_card_context=None, menu=None):
-        self.add_detail_card(code=code,
+    def add_main_card(self, details_object, extra_card_context, code='main'):
+        menu = self.get_details_menu(details_object=details_object)
+        title = self.get_details_title(details_object=details_object)
+        group_type = self.get_group_type(details_object=details_object)
+        show_created_modified_dates = self.get_show_created_modified_dates(details_object=details_object)
+        extra_card_kwargs = self.get_extra_card_kwargs(details_object=details_object)
+        template_name = self.get_card_template()
+        card = self.add_card(title=title,
+                             code=code,
+                             details_object=details_object,
+                             menu=menu,
+                             show_created_modified_dates=show_created_modified_dates,
+                             group_type=group_type,
+                             datatable_model=self.datatable_model,
+                             extra_card_context=extra_card_context,
+                             template_name=template_name,
+                             **extra_card_kwargs)
+        return card
+
+    def add_card(self, code=None, details_object=None, title=None, menu=None, template_name=None,
+                 group_type=CARD_TYPE_STANDARD,
+                 show_created_modified_dates=False, extra_card_context=None, **extra_card_kwargs):
+        if code is None:
+            code = random_string()
+
+        card = self.card_cls(request=self.request,
+                             view=self,
+                             code=code,
                              title=title,
                              menu=menu,
-                             group_type=self.CARD_TYPE_HTML,
+                             show_created_modified_dates=show_created_modified_dates,
+                             group_type=group_type,
+                             details_object=details_object,
                              extra_card_context=extra_card_context,
-                             template_name=self.template_name)
-        html = render_to_string(template_name, context)
-        self.detail_cards[code]['html'] = html
+                             template_name=template_name,
+                             **extra_card_kwargs)
+
+        self.cards[code] = card
+        if group_type == CARD_TYPE_STANDARD:
+            self.get_details_data(card=card, details_object=details_object)
+        return card
+
+    def _render_cards(self):
+        data = ""
+        for card_key, card in self.cards.items():
+            data += card.render()
+        return data
+
+    def get_details_data(self, card, details_object):
+        pass
 
 
 class CardList(AjaxHelpers, MenuMixin, CardListMixin):
